@@ -29,11 +29,10 @@ Description:    A modular screen saver and locker for the X Window System.
                 Highly customizable: allows the use of any program that
                 can draw on the root window as a display mode.
                 Comes with more than 80 display modes.
-                Home page: http://www.jwz.org/xscreensaver/
 Keywords:       screen saver, screen lock, lock, xlock, X11
 Author:         jwz@jwz.org (Jamie Zawinski)
 Maintained-by:  jwz@jwz.org (Jamie Zawinski)
-Primary-site:   ftp.x.org /contrib/applications/
+Primary-site:   http://www.jwz.org/xscreensaver/
                 $TAR_SIZE xscreensaver-$VERSION.tar.gz
                 $README_SIZE  xscreensaver.README
                 $LSM_SIZE   xscreensaver.lsm
@@ -41,6 +40,10 @@ Alternate-site: sunsite.unc.edu /pub/Linux/X11/screensavers/
                 $TAR_SIZE xscreensaver-$VERSION.tar.gz
                 $README_SIZE  xscreensaver.README
                 $LSM_SIZE   xscreensaver.lsm
+Alternate-site: ftp.x.org /contrib/applications/
+                $TAR_SIZE xscreensaver-$VERSION.tar.gz
+                $README_SIZE  xscreensaver.README
+                $LSM_SIZE   xscreensaver.lsm
 Platforms:      Linux, Irix, SunOS, Solaris, HPUX, AIX, FreeBSD, NetBSD,
                 BSDI, SCO, OSF1, Ultrix, VMS.
                 Requires X11 and ANSI C.
